#!/usr/bin/perl -w -I INSTALLDIR
#
#    getwmail - perl script to get mail from web-based inboxes.
#    Copyright (C) 2002 Doug Rinckes <doug.rinckes@opencast.mine.nu>
#    Based on GotMail which is Copyright (C) 2000 Peter Hawkins 
#    <peterhawkins@ozemail.com.au>.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# push the installation directory into the include array
# before installation this reads INSTALLDIR - the installation script substitutes
# the real install directory

use strict;
use English;
use URI::Escape;
use Mail::Sender;

use config;
use utils;
# now get the packages for the different web services
use mailcom;
use hotmail;

# result of email checks
my $check_result = 0;

# parse the configuration options
config::parse_config();

# output the introduction text
utils::intro_text();

# clear log files and temporary files
utils::remove_files();
utils::remove_tempfiles();

# check that all the programs we need are present
utils::check_programs();

# for each service...
if ($config::service eq "m") {
	if ($config::check_only eq 1) {
		# if we are only checking for mail
		$check_result = mailcom::check_new_mail(mailcom::login_to_webmail());
	} else {
		# otherwise process the mail
		mailcom::process_folder(mailcom::login_to_webmail());
	}
} elsif ($config::service eq "h") {
	if ($config::check_only eq 1) {
		$check_result = hotmail::check_new_mail(hotmail::login_to_webmail());
	} else {
		hotmail::process_folder(hotmail::login_to_webmail());
	}
}

# remove temporary files again
utils::remove_tempfiles();

exit($check_result);
