# hotmail.pm
# routines for communicating with the hotmail service
package hotmail;

use strict;


# Do the login process - log in until we have the URL of the inbox and return it.
sub login_to_webmail()
{
	my($page) = "";				# text string of entire page
	my($next_url) = "";			# store the next url to go to
	my($params);				# paramters for the login
	my($login) = $config::login;		# user login name
	my($domain);				# mail domain
	my($password) = $config::password;	# users password
	my($hostname);
	my($attempt) = 0;


	# split the users login into the login and the domain
	($login,$domain) = split(/@/,$login);

	while (!($page =~ m/passwordform/i) && ($attempt < 5)) {
		# first of all get the main index page for the site
		utils::show_text("Getting hotmail index page...\n",1);
		$page = join "", utils::get_page("http://www.hotmail.msn.com/", "", 1, 1, 0);
		utils::debug_to_file("01.home",$page,2);
		$attempt ++;
	}

	# the login submission URL is indicated by passwordform
	if ($page =~ m/<form(.*)name=\"passwordform\"(.*)action=\"(\S+)\"(.*)>/i) {
		$next_url = $3;
	}

	if ($next_url eq "") {
		utils::exit_and_send_failure("Hotmail's page structure has changed (cannot find the login form)\n",$page);
	}

	# add on the parameters for logging in
	$params = "login=" . URI::Escape::uri_escape("$login", "^A-Za-z0-9") . "\&passwd=" . URI::Escape::uri_escape($password, "^A-Za-z0-9");
	$params .= "\&domain=$domain\&svc=mail";
	$params .= "\&";
	$params .= utils::extract_hidden_input($page);

	# now log in...
	utils::show_text("Logging in...\n",0);
	$page = join "", utils::get_page($next_url, $params, 1, 1, 0);
	utils::debug_to_file("02.login",$page,2);


	# there is a bunch of redirection pages - it's easier (safer?) to do this in a loop
	# although it _might_ be endless it hasn't in practice.....
	#while ($page =~ m/top.location.replace\(\"(\S+)\"\)/i) {
	#while ($page =~ m/http-equiv=\"refresh\".*url=(\S+)\">/i) {
	$attempt = 0;
	while (!($page =~ m/logout/i) && ($attempt < 10)) {
		$page =~ m/http-equiv=\"refresh\".*url=(\S+)\">/i;
		$next_url = $1;

		$page = join "", utils::get_page($next_url, $params, 1, 1, 0);
		utils::debug_to_file("03.$attempt.folders",$page,2);

		if ($page =~ m/re-enter your password/i) {
			# at the password confirmation page
			if ($page =~ m/<form(.*)name=\"passwordform\"(.*)action=\"(\S+)\"(.*)>/i) {
				$next_url = $3;
				$params = "passwd=" . URI::Escape::uri_escape($password, "^A-Za-z0-9");
				$params .= "\&";
				$params .= utils::extract_hidden_input($page);
				$page = join "", utils::get_page($next_url, $params, 1, 1, 0);
				utils::debug_to_file("02.$attempt.login",$page,2);
			}
		}
		$attempt ++;
	}

	utils::show_text("Should have the folders page now\n",1);
	
	# the only place the hostname is given on the folders page is the logout link
	# we want to get it because we might have been redirected away from the original
	# host.
	if ($page =~ /(.*)\"(\S+)logout/i) {
		$hostname = utils::extract_host($2);
	} else {
		utils::exit_and_send_failure("Hotmail login gone wrong - try again later\n",$page);
	}

	utils::show_text("Using hotmail server $hostname\n",1);

	$next_url = "";
	$params = "";

	# find the link to the users inbox
	if ($page =~ m/\"(\S+HoTMaiL)\?(\S+)\"><font.*>Inbox/i) {
		$next_url = "$hostname" . "$1";
		$params = $2;
	} elsif ($page =~ /unavailable/i) {
		utils::exit_and_send_failure("Hotmail is reporting that your account is temporarily unavailable. Please try again later.",$page);
	} else {
		utils::exit_and_send_failure("Hotmail's page structure has changed (cannot find Inbox)\n",$page);
	}

	return("$next_url\?$params");
}


# retrieve an attachment, save to a file and return the filename
sub retrieve_attachment($$)
{
	my($url,$name) = @_;
	my($param);
	my($page);
	my($referer);
	my($filename);
	my($file_handle);


	if ($url eq "") {
		utils::exit_and_send_failure("Attachment with null url is illegal - the page format may have changed\n","");
	}

	$referer = $url;
	utils::show_text("Retrieving attachment\n",1);
	$page = join "", utils::get_page($url,"",1,1,0);
	utils::debug_to_file("06.attachment_01",$page,2);

	if ($page =~ /onclick=\"newattachwin\(\'(\S+)\?(\S+)\'/i) {
		$url = "$1\?$2";
		$page = join "", utils::get_page_extra("$url","","$referer", 1,1,0,"");
		utils::debug_to_file("06.attachment_02",$page,2);
	}
	$referer = $url;
	if ($page =~ /attachment.*src=\"(\S+)\?(\S+)\"/i) {
		$url = "$1\?$2";
		$filename = utils::return_temp_name("/var/tmp",$name);
		utils::get_page_extra("$url","","$referer", 1,1,-1,$filename);
		utils::show_text("Attachment stored in file $filename\n",2);
	}

	return($filename);
}


sub clean_up_message_body($)
{
	my($message) = @_;

	if (index($message,"<SCRIPT>") == 0) {
		# HTML
		# remove everything up to (and including) the </SCRIPT> tag
		$message = substr($message,index($message,"</SCRIPT>") + 10);
		# remove everything AFTER the LAST <font tag
		$message = substr($message,0,rindex($message,"<font"));
	} elsif (index($message,"<pre>") == 0) {
		# Text
		$message = substr($message,6);
		# remove everything AFTER the LAST </pre tag
		$message = substr($message,0,rindex($message,"</pre"));
		# clean it up even more
		$message =~ s/ <BR> //g;
		$message =~ s/ <P> /\n\n/g;
		$message =~ s/&amp;/&/g;
		$message =~ s/&lt;/</g;
		$message =~ s/&gt;/>/g;
		$message =~ s/&quot;/\"/g;
		$message =~ s/<[^>]*>//g;
	}
	return($message);
}
	


# forward the email and delete it
sub process_message($$)
{
	my($page,$hostname) = @_;
	my(@page_lines);
	my($m_from) = "";
	my($m_to) = "";
	my($m_cc) = "";
	my($m_subject) = "";
	my($m_date) = "";
	my(@email_page);
	my($body_index) = 3;	# the segment of @email_page with the message body
	my($delete_index) = 2;	# the segment of @email_page with the link to the delete button
	my($message_body);
	my($attachment) = "";
	my($attachment_file);
	my(@attachment_list);

	my(@inbox);
	my(@temp_array);
	my($post_url);
	my($input_fields);
	my($counter);

	# split the page up by lines
	@page_lines = split(/\n/,$page);
	# split the email up into the different divisions
	# typically three - one for the page header and info, one for the text of the message, and one for the footer etc.
 	@email_page = split(/<div>|<\/div>/i, $page);

	# now we're going to zip through the message examining it for the headers
	for ($counter = 0; $counter <= $#page_lines; $counter ++) {
		# get the from address
		if ((index($page_lines[$counter],"id=\"From\"") >= 0) && ($m_from eq "")) {
			$page_lines[$counter + 1] =~ />(.*)<\/font/;
			$m_from = $1;
			$m_from =~ s/&lt;/</g;
			$m_from =~ s/&gt;/>/g;
			$m_from =~ s/&quot;/\"/g;
		}
		# get the to address
		if ((index($page_lines[$counter],"<b>To") >= 0) && ($m_to eq "")) {
			$page_lines[$counter + 4] =~ />(.*)<\/font/;
			$m_to = $1;
			$m_to =~ s/&lt;/</g;
			$m_to =~ s/&gt;/>/g;
			$m_to =~ s/&quot;/\"/g;
		}
		# get the CC list
		if ((index($page_lines[$counter],"<b>CC") >= 0) && ($m_cc eq "")) {
			$page_lines[$counter + 4] =~ />(.*)<\/font/;
			$m_cc = $1;
			$m_cc =~ s/&lt;/</g;
			$m_cc =~ s/&gt;/>/g;
			$m_cc =~ s/&quot;/\"/g;
		}
		# get the subject
		if ((index($page_lines[$counter],"<b>Subject") >= 0) && ($m_subject eq "")) {
			$page_lines[$counter + 4] =~ />(.*)<\/font/;
			$m_subject = $1;
			$m_subject =~ s/&lt;/</g;
			$m_subject =~ s/&gt;/>/g;
			$m_subject =~ s/&quot;/\"/g;
		}
		# get the date
		if ((index($page_lines[$counter],"<b>Date") >= 0) && ($m_date eq "")) {
			$page_lines[$counter + 4] =~ />(.*)<\/font/;
			$m_date = $1;
		}
		# get the attachments
		if ((index($page_lines[$counter],"<b>Attachment") >= 0) && ($attachment eq "")) {
			$page_lines[$counter + 3] =~ />(.*)<\/font/;
			$attachment = $1;
		}
	}
			
	utils::show_text("Processing message\n",0);
	utils::show_text("Message from $m_from, subject $m_subject, date $m_date, cc $m_cc\n",1);

	# div section $body_index is the message body ...
	$message_body = $email_page[$body_index];

	$message_body = clean_up_message_body($message_body);

	# now, $message_body is what we will forward to the user

	utils::debug_to_file("06.message_body",
		"Message body - what gets sent:\n" . 
		$message_body,
		2);

	# if we have any, save and retrieve the attachments
	if ($attachment ne "") {
		foreach $attachment (split(/<a href=/,$attachment)) {
			if ($attachment ne "") {
				$attachment =~ /\"(.*)\" >(.*)<\/a/;
				utils::show_text("Processing attachment $2\n",1);
				push(@attachment_list, retrieve_attachment("$hostname$1",$2));
			}
			
		}
	}

	for ($counter = 3; $counter < $#email_page; $counter = $counter + 2) {
		$attachment = clean_up_message_body($email_page[$counter]);
		$attachment_file = utils::return_temp_name("/var/tmp","attachment" . (($counter - 3) / 2));
		utils::show_text("Processing inline attachment ($attachment_file)\n",1);
		utils::write_to_file($attachment_file,$attachment);
		push(@attachment_list,$attachment_file);
	}

	# convert the attachment list array to a single scalar
	$attachment = join "\n",@attachment_list;

	# forward the message and it's attachments
	utils::forward_message($message_body,$m_to,$m_from,$m_subject,$m_date,$m_cc,$attachment);
	utils::show_text("Forwarded message\n",0);

	# now delete the mail and move onto the next one
	if ($email_page[$delete_index] =~ /a href=\"http:(\S+)\" class(.*)Delete/) {
		utils::show_text("Deleting message\n",1);
		$page = join "", utils::get_page("http:$1", "", 1, 1, 0);
	} else {
		utils::show_text("Could not find delete button\n",0);
		$page = "";
	}

	return($page);
}

# Get the messages from the folder at url
sub process_folder($)
{
	my($url) = @_;
	my(@page_array);
	my($page);
	my(@messages);
	my($hostname);
	my($message_counter) = 0;
	my($params);


	utils::show_text("Processing Inbox...\n",0);

	$hostname = utils::extract_host($url);

	if ($url =~ m/(\S+)\?(\S+)/) {
		$url = $1;
		$params = $2;
	}

	@page_array = utils::get_page("$url", "$params", 1, 1, 0);
	$page = join "", @page_array;

	utils::debug_to_file("04.inbox",$page,2);

	# Redo the list on a table row by table row basis
	@messages = grep(/getmsg/i, @page_array);
	if ($#messages == -1) {
		utils::show_text("Inbox empty\n",0);
		return();
	}

	# Get the first message in this folder...
	if ($messages[0] =~ /\"(\S+)\?(\S+)\"/) {
		$url = "$hostname" . "$1" . "\?" . $2;
		# POST method doesn't work - don't specify the params separately
		@page_array = utils::get_page($url,"",1,1,0);
		$page = join "", @page_array;
		utils::debug_to_file("05.message.$message_counter",$page,2);
		$message_counter ++;
		while ((grep(/id=\"From\"/,$page)) || (grep(/msgFromName/,$page))) {
			$page = process_message($page,$hostname);
			utils::debug_to_file("05.message.$message_counter",$page,2);
			$message_counter ++;
		}
	}
}


sub check_new_mail($)
{
	my($url) = @_;
	my(@page_array);
	my($page);
	my($hostname);
	my($params);

	utils::show_text("Checking Inbox...\n",0);

	$hostname = utils::extract_host($url);

	if ($url =~ m/(\S+)\?(\S+)/) {
		$url = $1;
		$params = $2;
	}

	@page_array = utils::get_page("$url", "$params", 1, 1, 0);
	$page = join "", @page_array;

	utils::debug_to_file("04.inbox",$page,2);

	if ($page =~ /newmail.gif/) {
		utils::show_text("You have unread mail\n",0);
		return(0);
	} elsif ($page =~ /getmsg/) {
		utils::show_text("You have mail\n",0);
		return(0);
	} else {
		utils::show_text("No new mail for you\n",0);
		return(1);
	}
}


1;
