# mailcom.pm
# routines for communicating with the mail.com and iname.com service
package mailcom;

use strict;



# Do the login process - log in until we have the URL of the inbox and return it.
sub login_to_webmail()
{
	my($page);				# text string of entire page
	my($next_url);				# store the next url to go to
	my($params);				# paramters for the login
	my($login) = $config::login;		# user login name
	my($password) = $config::password;	# users password

	# first of all get the main index page for the site
	utils::show_text("Getting mail index page...\n",1);
	$page = join "", utils::get_page("http://www.mail.com/", "", 1, 1, 0);
	utils::debug_to_file("01.home",$page,2);

	# the login submission URL is indicated by loginForm
	if ($page =~ m/<form(.*)action=\"(\S+)\"(.*)name=\"loginForm\"(.*)>/i) {
		$next_url = $2;
	}

	# add on the parameters for logging in
	$params = "action=login\&show_frame=Enter\&mail_language=us\&login=";
	$params .= URI::Escape::uri_escape("$login", "^A-Za-z."). "\&password=" . URI::Escape::uri_escape($password, "^A-Za-z");

	utils::show_text("Logging in...\n",0);

	# now log in...
	$page = join "", utils::get_page($next_url, $params, 1, 1, 0);
	utils::debug_to_file("02.login",$page,2);

	# did we manage to login OK?
	if (utils::check_page_for($page,"invalid username")) {
		utils::exit_and_send_failure("Mail.Com is reporting that the username or password are incorrect.",$page);
	} elsif (utils::check_page_for($page,"cannot connect to server")) {
		utils::exit_and_send_failure("Mail.Com's web front-end could not contact the mail server - retry.",$page);
	}
	# once logged in - get the link to show the folder list - outblaze.main
	if ($page =~ m/(.*)\"(\S+frontpage.main)\?(\S+)\"(.*)/i) {
		$next_url = $2;
		$params = $3;
	}  elsif ($page =~ m/(.*)\"(\S+outblaze.main)\?(\S+)\"(.*)/i) {
		$next_url = $2;
		$params = $3;
	} else {
		utils::exit_and_send_failure("Can't get the link to the folders page after logon.",$page);
	}

	if (($next_url eq "") || ($params eq ""))
	{
		utils::exit_and_send_failure("Mail.Com's page structure has changed; can't find the link to the folder list.",$page);
	}

	utils::show_text("Getting folder list...\n",1);

	# retrieve the page with the list of folders - Inbox, Sent Mail etc.  
	# this is so we can get the link to the Inbox
	$page = join "", utils::get_page($next_url, $params, 1, 1, 0);
	utils::debug_to_file("03.folders",$page,2);

	# how much of the quota is in use
	if ($page =~ m/you are using (.*) of your mail quota/i) {
		utils::show_text("$1 of your mail quota is in use\n",0);
	}

	# Find where the inbox is located... 
	$next_url = "";
	if ($page =~ m/http:\/\/(.*)folder=INBOX"/i) {
		$next_url = "http://" . substr($1,0,index($1,"\""));
		$next_url = "http://" . $1 . "folder=INBOX";
		
	} elsif (utils::check_page_for($page,"unavailable")) {
		utils::exit_and_send_failure("Mail.Com reports your account is temporarily unavailble.",$page);
	} else {
		utils::exit_and_send_failure("Can't find the link to the inbox (page layout may have changed)",$page);
	}
		
	if ($next_url eq "")
	{
		utils::exit_and_send_failure("Mail.Com's page structure has changed (cannot find Inbox)\n",$page);
	}
	return $next_url;
}


# retrieve an attachment, save to a file and return the filename
sub retrieve_attachment($$)
{
	my($url,$name) = @_;
	my($params);
	my($page);
	my($filename);
	my($file_handle);

	($url,$params) = split(/\?/,$url);

	if (($url eq "") || ($params eq "")) {
		utils::exit_and_send_failure("Attachment with null url or parameters is illegal - the page format may have changed\n","");
	}

	if ($config::verbosity > 0)
	{
		utils::show_text("Retrieving attachment\n",1);
	}
	$filename = utils::return_temp_name("/var/tmp",$name);

	utils::get_page_extra($url,$params,"",1,1,-1,$filename);

	return($filename);
}



sub clean_up_message_body($)
{
	my($message) = @_;
	my($leftover);

	# but it needs to be cleaned up a bit
	($message,$leftover) = split(/\/scripts\/mail\/mesg.mail\"/, $message);
	$message =~ s/^[^\n]*\n//;		# remove the first line
	$message =~ s/^[^\n]*\n//;		# remove the first line
	$message =~ s/^[^\n]*\n//;		# remove the first line
	$message =~ s/^[^\n]*\n//;		# remove the first line
	$message =~ s/^[^\n]*\n//;		# remove the first line

	$message =~ s/^[\n\s]+//;			# remove leading white space
	$message =~ s/\s+$//;			# remove trailing white space

	#$message =~ s/^<\/form>.*\n//i;		# remove the form entry
	#$message =~ s/^<pre>.*\n//i;		# remove a leading PRE tag
	$message =~ s/^<\/form>//i;		# remove the form entry
	$message =~ s/^<pre>//i;		# remove a leading PRE tag

	$message .= "\n\n";
	$message =~ s/[^\n]*mail.com\/scripts\/mail\/getattch[^\n]*\n//ig;
	$message =~ s/[^\n]*mail.com\/scripts\/mail\/getattach[^\n]*\n//ig;
	$message =~ s/\n*$//g;			# remove any attachment lines and the trailing newline

	# is it text or is it html? Assume that if it starts with a < 
	# and a char, that it's a tag and that this is a html message
	if (grep(/^<[A-Za-z!]/i,$message)) {
		# the message body is html
		$message =~ s/<xhtml>/<html>/ig;
		$message =~ s/<\/xhtml>/<\/html>/ig;
		$message = substr($message,0,rindex($message,"</html>"));
	} else {
		# the message body is text
		# clean it up even more
		$message =~ s/<\/PRE>/<\/pre>/ig;
		$message = substr($message,0,rindex($message,"</pre>"));
		$message =~ s/ <BR> //ig;
		$message =~ s/ <P> /\n\n/ig;
		$message =~ s/&amp;/&/g;
		$message =~ s/&lt;/</g;
		$message =~ s/&gt;/>/g;
		$message =~ s/&quot;/\"/g;
		$message =~ s/&nbsp;/ /g;
	}

	return($message);
}
	

# forward the email and delete it
sub process_message($)
{
	my($page) = @_;
	my($m_from) = "";
	my($m_to) = "";
	my($m_subject) = "";
	my($m_date) = "";
	my($m_cc) = "";
	my(@email_page);
	my($message_body);
	my($attachment);
	my(@attachment_list);
	my($delete_form);

	my(@inbox);
	my(@temp_array);
	my($post_url);
	my($input_fields);
	my($counter);
	my($trash);

	# split the email up into the different html tables
 	@email_page = split(/<table|<\/table>/, $page);

	# Find the From header:
	$m_from = substr($page, index($page, ">From:") + 6, 200);	# get after the From: field
	$m_from = utils::clean_header_field($m_from);

	$m_to = substr($page, index($page, ">To:") + 4, 200);	# get after the To: field
	$m_to = utils::clean_header_field($m_to);

	$m_cc = substr($page, index($page, ">Cc:") + 4, 200);	# get after the Cc: field
	$m_cc = utils::clean_header_field($m_cc);

	$m_subject = substr($page, index($page, ">Subject:") + 8, 200);	# get after the Subject: field
	$m_subject = utils::clean_header_field($m_subject);

	$m_date = substr($page, index($page, ">Date:") + 6, 200);	# get after the Date: field
	$m_date = utils::clean_header_field($m_date);

	utils::show_text("Processing message\n",0);
	utils::show_text("Message from $m_from, subject $m_subject, date $m_date, cc $m_cc\n",1);

	# The message is enclosed (roughly) by calls to javascript:closeMail at the start and end
	# although it requires significant cleaning up at the start and at the end
	#@email_page = split(/javascript:closeMail/,$page);
	@email_page = split(/msg_unflag_end/,$page);
	$message_body = clean_up_message_body($email_page[1]);

	# now, $message_body is what we will forward to the user
	utils::debug_to_file("06.message_body",
		"Message body - what gets sent:\n" . 
		$message_body . 
		"\n\n================================================\n\n" . 
		"Original section - before tag removal etc:\n" . 
		$email_page[1],
		2);

	# now save and retrieve the attachments
	foreach $attachment (grep(/getattch/,split(/\n/,$email_page[1]))) {
		if ($attachment =~ /<a href="(.*getattch.*)"><font.*>(.*)<\/font/i) {
			#$attachment =~ /.*"(.*getattch.*)"><FONT.*>(.*)<\/FONT/;
			# retrieve the attachment to a file, and store the file name in the array
			utils::show_text("Processing attachment $attachment\n",1);
			push(@attachment_list, retrieve_attachment($1,$2));
		}
	}
	foreach $attachment (grep(/getattach/,split(/\n/,$email_page[1]))) {
		if ($attachment =~ /<a href="(.*getattach.*)"><font.*>(.*)<\/font/i) {
			# retrieve the attachment to a file, and store the file name in the array
			utils::show_text("Processing attachment $attachment\n",1);
			push(@attachment_list, retrieve_attachment($1,$2));
		}
	}

	# convert the attachment list array to a single scalar
	$attachment = join "\n",@attachment_list;

	# forward the message and it's attachments
	utils::forward_message($message_body,$m_to,$m_from,$m_subject,$m_date,$m_cc,$attachment);

	# split the page into the different form areas
	@email_page = split(/<form/, $page);
	# the delete form is the second form on the page
	$delete_form = $email_page[2];
	# now use grep to get the post line
	@temp_array = grep(/POST/,$delete_form);
	$temp_array[0] =~ /\s+action=\"(\S+)\" method/;
	$post_url = $1;

	$input_fields = utils::extract_hidden_input($delete_form);
	$input_fields =~ s/delete_read=/delete_read=yes/;
	#$input_fields .= "\&folder_name=Drafts";
	utils::show_text("Deleting message...\n",1);
	utils::show_text("Delete params = $input_fields\n",2);

	# and then call to delete the mail and return the next one
	@inbox = utils::get_page($post_url,"$input_fields",1,1,0);
	$page = join "", @inbox;
	utils::debug_to_file("07.delete_response",$page,2);
	return($page);
}

# Get the messages from the folder at url
sub process_folder($)
{
	my($url) = @_;
	my(@page_array);
	my($page);
	my(@messages);
	my($message_url);
	my($message_counter) = 0;


	utils::show_text("Processing Inbox...\n",0);

	@page_array = utils::get_page("$url", "", 1, 1, 0);
	$page = join "", @page_array;

	utils::debug_to_file("04.inbox",$page,2);

	# Redo the list on a table row by table row basis
	@messages = grep(/:readPopUpMail/i, @page_array);
	if ($#messages == -1) {
		utils::show_text("Inbox empty\n",0);
		return();
	}

	# Get the first message in this folder...
	# the link to the message is contained between the FIRST two quote characters
	# can't use // to match these, since they start from the end of the string and
	# this string includes the subject, which may have ' characters in it

	# get the link from after the first '
	$message_url = substr($messages[0],index($messages[0],"'") + 1,500);
	# and then terminate the string at the first ' in it now (the second original ')
	$message_url = substr($message_url,0,index($message_url,"'"));
	
	if ($message_url ne "") {
		utils::show_text("Getting a message ($message_url) ...\n",1);
		@page_array = utils::get_page($message_url,"",0,1,0);
		$page = join "", @page_array;
		utils::debug_to_file("05.message.$message_counter",$page,2);
		$message_counter ++;
		while (grep(/From:/,$page)) {
			$page = process_message($page);
			utils::debug_to_file("05.message.$message_counter",$page,2);
			$message_counter ++;
		}
	}
}


sub check_new_mail($)
{
	my($url) = @_;
	my(@page_array);
	my($page);

	utils::show_text("Checking Inbox...\n",0);
	@page_array = utils::get_page("$url", "", 1, 1, 0);
	$page = join "", @page_array;

	utils::debug_to_file("04.inbox",$page,2);

	if ($page =~ /mesg.mail/) {
		$page =~ /pagehead.*inbox.*message.*, (\S+) unread/i;
		if ($1 > 0) {
			utils::show_text("You have unread mail\n",0);
		} else {
			utils::show_text("You have mail\n",0);
		}
		return(0);
	} else {
		utils::show_text("No mail for you\n",0);
		return(1);
	}
}


1;

