package utils;
use strict;
use POSIX qw(tmpnam);
use FileHandle;

my $getwmail_version = "0.10";
my $getwmail_date = "2002-11-28";
my $log = "/var/tmp/getwmail.log";
my $tmp_headers = tmpnam()."gotmail_headers";
my $tmp_cookies = tmpnam()."gotmail_cookies";
my $conf_curl;


# Various utility functions
sub intro_text()
{
    if ($config::verbosity >= 0)
    {
        print "getwmail v".$getwmail_version."    Copyright (C) 2002 Doug Rinckes\n";
	print "getwmail comes with ABSOLUTELY NO WARRANTY.\n";
	print "This is free software, and you are welcome to redistribute it\n";
	print "under certain conditions; see the file COPYING for details.\n\n";
    }
}

sub dispVersionText()
{
    print "Version information: getwmail v".$getwmail_version."   Date: ".$getwmail_date."\n";
}

# Display some text to the screen, and log it, if we are in debug mode.
sub show_text($$)
{
	my($text,$verbosity_threshold) = @_;

	if ($config::verbosity >= $verbosity_threshold) {
		print $text;
		open(OUT,">>$log") || return;
		print OUT $text;
		close(OUT);
    	}
}


# dump the contents to the named file
sub debug_to_file($$$)
{
	my ($filename,$contents,$verbose_threshold) = @_;
	my ($handle);

	if ($config::verbosity >= $verbose_threshold) {
		write_to_file("/tmp/$filename",$contents);
		show_text("Debugging to $filename\n",$verbose_threshold);
	}
}


# write passed data to passed file
sub write_to_file($$)
{
	my($filename,$data) = @_;
	my($handle);

	open(HANDLE,">$filename");
	print HANDLE $data;
	close(HANDLE);
}


# Clean up any temporary files
sub remove_tempfiles()
{
	if (-e $tmp_headers) { unlink($tmp_headers); }
	if (-e $tmp_cookies) { unlink($tmp_cookies); }
}

sub remove_files()
{
	if (-e $log) 	     { unlink($log); }
}

# Check all the required programs are installed.
sub check_programs()
{
	show_text("System version is: ".$main::OSNAME."\n",2);
	show_text("Perl version is:   ".$]."\n",2);

	if (-x "./curl") { 
		$conf_curl = "./curl"; 
	} else { 
		$conf_curl = "curl"; 
	}
	show_text("Curl version is:   ".`$conf_curl --version`."\n",2); 

	# create an empty cookies file
	open(FILE_HANDLE,">$tmp_cookies") || die "Cannot create session cookie file\n";
	close(FILE_HANDLE);

}


# get the cookies from the last request, and merge them into the session cookie file
sub save_cookies()
{
	my(@cookies);
	my($cookie_str);

	# first, read in the existing cookies
	open(FILE_HANDLE,"$tmp_cookies") || return;
	while (<FILE_HANDLE>) {
		push(@cookies, $_);
	}
	close(FILE_HANDLE);

	open(FILE_HANDLE,"$tmp_headers") || return;
	while (<FILE_HANDLE>)
	{
		if (m/^Set-Cookie: ([A-Za-z0-9_]*)=/i) {
			# Remove any previous setting for this cookie
			@cookies = grep { !/$1/ } @cookies;

			# Make cookies globally applicable. Evil. =)
			# In fact so evil it breaks attachment downloading from hotmail.  Go figure.
			#s/domain=([^\;]+)\;/domain=.com\;/;

			push(@cookies, $_);
		}
	}
	close(FILE_HANDLE);

	# Dump the cookie list into the session file so curl can read it.
	open(FILE_HANDLE,">$tmp_cookies") || return;
	$cookie_str = join "", @cookies;
	print FILE_HANDLE $cookie_str;
	close(FILE_HANDLE);
	show_text("Session cookies are now:\n$cookie_str\n",2);
}



# Fetch a given page using curl
# The parameters taken are the URL, any data to be posted in a POST,
# whether we are to follow HTTP redirects, and whether we should send and receive
# cookies, whether we should only get the headers for this page and not the body.
#
# headers_flag = -1, get the body only, =0 get both, =1 get headers only


sub get_page($$$$$)
{
	my($url, $params, $follow_forward, $cookies, $headers_only) = @_;

	return(get_page_extra($url,$params,"",$follow_forward,$cookies,$headers_only,""));
}

sub get_page_extra_stub($$$$$$$)
{
	my($url, $params, $referer, $follow_forward, $cookies, $headers_only, $filename) = @_;

	return(get_page_extra($url,$params,$referer,$follow_forward,$cookies,$headers_only,$filename));
}


# extra options version of the getpage script
# you can pass a call either all in the url
#	url = "http://foo.bar/cgi-bin/wonk?a=1&b=2"
# or split between the url and params:
#	url = "http://foo.bar/cgi-bin/wonk"
#	params = "a=1&b=2"
# the difference is that the first will do an HTTP GET, 
# and the second an HTTP POST.
# hotmail likes everything as GETs, mail.com likes everything
# as POSTs - they are not interchangeable.
sub get_page_extra($$$$$$$)
{
	my($url, $params, $referer, $follow_forward, $cookies, $headers_only, $filename) = @_;
	my($options);
	my($cmdline);
	my(@tmp_page);
	my($tries);

	show_text("get_page: $url\n",2);
	
	# Set up the options string...
	$options = "";
	if ($config::proxy)        { $options .= "--proxy ". $config::proxy . " "; }
	if ($config::proxy_auth)   { $options .= "--proxy-user ". $config::proxy_auth . " "; }
	if ($cookies != 0)      { $options .= "-b $tmp_cookies " }
	if ($params ne "")      { $options .= "--data \"$params\" " }
	if ($headers_only == 0) { $options .= "-i " }
	if ($headers_only == 1) { $options .= "-I " }
	if ($referer ne "")     { $options .= "-e \"$referer\" " }
	if ($filename ne "") 	{ $options .= "-o \"$filename\" " }
	if ($follow_forward != 0)  { $options .= "-L " }
	$options .= "-s -S ";
	$options .= "-m 120 ";

	#$cmdline = "$conf_curl \"$url\" $options -D $tmp_headers -A \"Mozilla/4.73 [en] (Win98; I)\"";
	$cmdline = "$conf_curl \"$url\" $options -D $tmp_headers -A \"Mozilla/4.0 (compatible; MSIE 5.01; Windows 95)\"";
	show_text("command line: $cmdline\n",2);
	$tries = 1;
	@tmp_page =  `$cmdline`;

	# Retry at most $config::retry_limit times if we fail.
	while (!@tmp_page && ($headers_only == 1) && ($tries++ <= $config::retry_limit)) {
		show_text("An error was encountered getting the page, retrying [$tries/$config::retry_limit]...\n",0);
		@tmp_page = `$cmdline`;
	}
	if (!@tmp_page && ($headers_only == 1) && ($tries > $config::retry_limit)) {
		die("An error was encountered getting the page. Command was $cmdline"); 
	}

	if ($cookies) { save_cookies(); }

	return @tmp_page;
}


# just send a message
sub send_a_message($$$$$) 
{
	my($body,$notice,$from,$subject,$attachments) = @_;
	my($mail_handle);
	my($filename);
	my($origname);

	$mail_handle = new Mail::Sender {from => "$from",smtp => "$config::smtp_server"};
	$mail_handle = $mail_handle->OpenMultipart({to=> "$config::forward_to",subject=> "$subject"});
	# at this point, mail_handle will either be a valid mail handle or a number less than zero
	# indicating that something has gone badly wrong.
	if (scalar $mail_handle < 0) {
		die "Message sending failed: $Mail::Sender::Error";
		exit(1);
	}
	if (grep(/^<[A-Za-z!]/i,$body)) {
		# if the body looks like html...
		$mail_handle->Body(0,0,"text/html");
	} else {
		# it is text.  i hope.
		$mail_handle->Body();
	}
	$mail_handle->Send($body);
	# if we have a notice (the getwmail notice), attach it
	if ($notice ne "") {
		$mail_handle->Body();
		$mail_handle->Send($notice);
	}
	# and attach all the attachments (if there are any)
	foreach $filename (split(/\n/,$attachments)) {
		$origname = substr($filename,rindex($filename,"/") + 1);
		show_text("Attaching file $filename\n",2);
		$mail_handle->Attach({
			encoding => 'Base64',
			description => $origname,
			disposition => "attachment; filename = $origname",
			file => $filename
			});
	}
	# send!
	$mail_handle->Close();
}


sub forward_message($$$$$$$)
{
	my($message_body,$m_to,$m_from,$m_subject,$m_date,$m_cc,$attachments) = @_;
	my($notice);
	my($filename);


	# if the message body is html, need to set the content type appropriately
	#if (grep(/^<!doctype/i,$message_body)) {

	$notice  = "This mail was forwarded by getwmail version $getwmail_version.\n";
	$notice .= "Some formatting may be damaged, and it is possible that < and >\n";
	$notice .= "characters have been removed, but all attachments should be fine.\n\n";
	$notice .= "The details of the original mail are:\n";
	$notice .= "	To:		$m_to\n";
	$notice .= "	From:		$m_from\n";
	$notice .= "	Subject:	$m_subject\n";
	$notice .= "	Date:		$m_date\n";
	$notice .= "	CC:		$m_cc\n";

	utils::send_a_message($message_body,$notice,$m_from,$m_subject,$attachments);

	foreach $filename (split(/\n/,$attachments)) {
		unlink($filename);
	}
	
	if ($config::speed_limit) { sleep(1); }
}

# something that requires dieing has happened - so we'll send an error message to the 
# forwarding address and attach whatever data we have.
sub exit_and_send_failure($$)
{
	my ($error_message,$supplementary_data) = @_;
	my ($mail_body);
	my ($temp_file);
	my ($localhostname);

	# if no email forwarding to address defined...
	if ($config::forward_to eq "") {
		die "$error_message";
		exit(1);
	}

	# so we have an email address - send the error message and all the params to it

	$mail_body = "$error_message\n\n";
	$mail_body .= "getwmail was called with:\n";
	$mail_body .= "	login = $config::login\n";
	$mail_body .= "	password = XXXXXXXXXX\n";
	$mail_body .= "	forward_to = $config::forward_to\n";
	$mail_body .= "	smtp_server = $config::smtp_server\n";
	$mail_body .= "	service = $config::service\n";
	$mail_body .= "	proxy = $config::proxy\n";
	$mail_body .= "	proxy_auth = $config::proxy_auth\n";
	$mail_body .= "	speed_limit = $config::speed_limit\n";
	$mail_body .= "	retry_limit = $config::retry_limit\n";
	$mail_body .= "	verbosity = $config::verbosity\n";
	$mail_body .= "	check_only = $config::check_only\n";

	# put the supplementary data into a file
	$temp_file = return_temp_name("/var/tmp","error_data");
	open(TFILE,">$temp_file") || die "Could not create temporary file $temp_file";
	print TFILE $supplementary_data;
	close(TFILE);

	# get the local hostname - some (most?) mail servers require a domainname on the sender address
	# so we will use this.  gethostent may cause problems if running on a non-unix server.
	# oh well.
	($localhostname) = gethostent();
	utils::send_a_message($mail_body,"","getwmail\@$localhostname","getwmail had an error","$temp_file\n");

	unlink($temp_file);
	die "$error_message";
	exit(1);

}

# generate a temporary file with a name similar to that which is passed
sub return_temp_name($$)
{
	my($dir, $name) = @_;
	my($counter);
	my($newname);

	$newname = "$dir/$name";
	$counter = -1;
	while (new FileHandle "<$newname") {
		$counter ++;
		$newname = sprintf("%s/%03d_%s",$dir,$counter,$name);
	}
	return($newname);
}


# extract the hidden input fields and values from an HTML page
sub extract_hidden_input($)
{
	my ($page) = @_;
	my (@params);
	my ($param);
	my ($name, $value);
	

	foreach $param (grep(/input type=\"hidden\"/,split(/\n/,$page))) {
		if ($param =~ m/name=\"(\S+)\"/i) {
			$name = $1;
		}
		if ($param =~ m/value=\"(\S+)\"/i) {
			$value = $1;
		} else {
			$value = "";
		}
		push(@params,"$name=". URI::Escape::uri_escape($value,"^A-Za-z"));
	}
	$param = join "\&",@params;
	return($param);
}


# work out the method and hostname from a URL
sub extract_host($)
{
	my($url) = @_;
	my(@parts);

	@parts = split(/\//,$url);

	return($parts[0] . "//" . $parts[2]);
}


# clean up a mail header field from the email
sub clean_header_field($)
{
	my($field) = @_;
	my($trash);

	$field =~ s/\n\n/\n/g;					# remove double feeds (might be here)
	($trash,$field,$trash) = split(/\n/,$field);		# we only want the second line
	$field =~ s/<.*?>//g;					# remove all the html tags
	$field =~ s/^\s//g;					# strip leading whitespace
	$field =~ s/\s$//g;					# strip trailing whitespace
	$field =~ s/&lt;/</g;
	$field =~ s/&gt;/>/g;
	$field =~ s/&quot;/\"/g;
	$field =~ s/&#34;/\"/g;
	return($field);
}


sub check_page_for($$)
{
	# check a page for a string
	# if the string is in a comment, it counts as not found
	# return true if found, false if not
	
	my ($page,$lookfor) = @_;
	my ($line);
	my ($incomment) = 0;
	my ($found) = 0;

open(FOO,">/tmp/pagefoo");
print FOO "$page\n\n";
close(FOO);
	foreach $line (split(/\n/,$page)) {
		if (index($line,"<!--") >= 0) {
			# we are in a comment
			$incomment = 1;
		}
		if (index($line,"-->") >= 0) {
			# we are not in a comment
			$incomment = 0;
		}
		if (!$incomment) {
			# if we're not in a comment...
			if ($line =~ m/$lookfor/i) {
				$found = 1;
			}
		}
	}
	return($found);
}



1;
